<?php

function wp_beifen_process_ajax_request($request_options)
{
	// Get options
	$options = get_option(WP_BEIFEN_OPTIONS);
	
	// A shortcut for DIRECTORY_SEPARATOR
	if(!defined(DS))
	{
		define('DS', DIRECTORY_SEPARATOR);
	}
	
	// Defines for recursive function use
	define('WP_BEIFEN_DIR', $options['plugin_backup_directory']);
	define('WP_BEIFEN_CURRENT_BACKUP', WP_BEIFEN_DIR . $request_options['backup_name'] . DS);
	
	// Load required class definitions and create instances
	require_once($options['plugin_location'] . 'classes' . DS . 'file.php');
	require_once($options['plugin_location'] . 'classes' . DS . 'database.php');
	$bkp_db = new XinitBackupDatabaseHelper();
	$bkp_file = new XinitBackupFileHelper();

	// Check for existing backups with same name
	if($bkp_db->existingBackup($request_options['backup_name']))
	{
		$result['status'] = __("Error", WP_BEIFEN_DOMAIN);
		$result['message'] = __("There is already a backup with this name. Please change the name!", WP_BEIFEN_DOMAIN);
		return $result;
	}
	
	// Check if main backup directory is writable
	if(!is_writable(WP_BEIFEN_DIR))
	{
		$result['status'] = __("Error", WP_BEIFEN_DOMAIN);
		$result['message'] = __("The backup folder is not writable. Please check write-permissions!", WP_BEIFEN_DOMAIN);
		return $result;
	}
	
	// Check if backup destination directory is writable and existing
	if(!@mkdir(WP_BEIFEN_CURRENT_BACKUP) && !file_exists(WP_BEIFEN_CURRENT_BACKUP))
	{
		$result['status'] = __("Error", WP_BEIFEN_DOMAIN);
		$result['message'] = __("Could not create backup folder. Please check write-permissions!", WP_BEIFEN_DOMAIN);
		return $result;
	}
	
	switch($request_options['backup_type'])
	{
		case 'Full':
			$bkp_file->writeTextToFile(WP_BEIFEN_CURRENT_BACKUP . 'database_dump.sql', $bkp_db->getMysqlDump());
			$bkp_file->copyFolderRec(ABSPATH,WP_BEIFEN_CURRENT_BACKUP . 'root' . DS,WP_BEIFEN_DIR);
			break;
		case 'Files':
			$bkp_file->copyFolderRec(ABSPATH,WP_BEIFEN_CURRENT_BACKUP . 'root' . DS,WP_BEIFEN_DIR);
			break;
		case 'DB':
			$bkp_file->writeTextToFile(WP_BEIFEN_CURRENT_BACKUP . 'database_dump.sql', $bkp_db->getMysqlDump());
			break;
		default:
			$result['status'] = __("Error", WP_BEIFEN_DOMAIN);
			$result['message'] = __("This is not a valid backup type!", WP_BEIFEN_DOMAIN);
			return $result;
	}
	
	// Zip backup, if requested
	if($request_options["compress_backup"]=='Yes')
	{
		require_once($options['plugin_location'] . 'classes' . DS . 'zip.php');
		$zipper = new XinitBackupZipHelper();
		$zipper->createZip(WP_BEIFEN_CURRENT_BACKUP . $request_options['backup_name'].'.zip', WP_BEIFEN_CURRENT_BACKUP);
	}
	
	// If everythings ok ...
	$zipped = ($request_options["compress_backup"]=='Yes')  ? "TRUE" : "FALSE";
	$bkp_db->insertNewBackupEntry($request_options['backup_name'], WP_BEIFEN_CURRENT_BACKUP, $request_options['backup_type'], $zipped);	
	$result['status'] = __("Success", WP_BEIFEN_DOMAIN);
	$result['message'] = __("Backup has been successfully created!", WP_BEIFEN_DOMAIN);
	return $result;
}

?>